/**********************************************************************

	Command:  CHTEXT  -- A Dialog Based Text Modification Program

	Programmed by Kang, Hsin-Min.
	Release at Feb 19, 1998
	Revised at Mar 31, 1998 -- Fix "[Pickup] not effective."
	Revised at Apr 22, 1998 Include Japenese Text in SJIS codes
	Revised at Aug 05, 1998 Modify Japenese Texts.
	Revised at Oct 20, 1998 Fix such that it will be compatible
				with current release of DDSTYLE

	Copyright (c) 1998, TCAM Development House, Taipei
	All rights Reserved.

	Status: TwinCAD Standard Command, with full support from
		TCAM Development House.

   Comments:

	This program will call DDSTYLE to build style table and to
	handle general style modification setting. The DDSTYLE has
	also been revised for this purpose. So, the user must update
	both programs.

	The CHTEXT will work in two modes:

	 -- Single Text Modification: The operator may change everything
	    about the text, including the text contents.

	 -- Multiple Text Modification: The operator may selective change
	    the text style and attribute of these selected text, excluding
	    their individual text contents.

***********************************************************************/
#define BOOL	int
#define TRUE	1
#define FALSE	0
#define EOF	-1

#define MARKON	0x8000		/* DOS version compatible marking value */
#define MARKOFF 0x80ff		/* DOS version compatible marking value */
#define FTEXT		0x40

#define DEBUG	0

#define OBM_COMBO	"#32738"

#define BIG5		0x404
#define GB2312		0x804
#define SJIS		0x411
#define KOREAN		0x412
#define ASCII		0x409

#define BIGSHX		3
#define SHAPEFILE	4

#define DOCANCEL	-1
#define DOIT		0
#define DOADDTEXT	1
#define DOPICKUP	2
//
#define SECTIONNAME	"CHText"
#define MAINWINDOW	"MainWindow"
//
#define szStdSname	@TXTSTYLE
#define szExtSname	@TXTESTYLE
#define CharWidth	@TXTWIDTHF
#define CharHeight	@TXTHEIGHT
#define CharSpace	@TXTCSPACE
#define CharObAngle	@TXTOBANGLE
#define RowSpace	@TXTROWDIST
#define CharRot90	@TXTROTCCW
#define CharObx 	@TXTOBX

#define szStdSvar	"TXTSTYLE"
#define szExtSvar	"TXTESTYLE"
#define szCharWidth	"TXTWIDTHF"
#define szCharHeight	"TXTHEIGHT"
#define szCharSpace	"TXTCSPACE"
#define szCharObAngle	"TXTOBANGLE"
#define szRowSpace	"TXTROWDIST"
#define szCharRot90	"TXTROTCCW"
#define szCharObx	"TXTOBX"

#define FILEMAGICID	0x1a434241L
#define STYLENAMESIZE	12
#define FONTNAMESIZE	40
#define MAXBUILTIN	4
#define FILENAMESIZE	64	/* Maximum possible size of full pathname */
#define MAXSTDFONT	256	/* Maximum # of Standard Font allowed */
#define MAXSTDFONT1	(MAXSTDFONT-MAXBUILTIN)
#define MAXEXTFONT	256	/* Maximum # of Extended Font allowed */
#define MAXEXTFONT1	(MAXEXTFONT-MAXBUILTIN)
#define MAXDIR		64	/* Maximum # of Search Directory */
#define FNAMESIZE	14
#define MAXSRCHFONT	256	/* Maximum # of fonts from single directory */
#define INDICATORSIZE	8	/* diviser of the font height */
#define INDICATORCOLOR	0x8e
#define DATACOLOR	0xf1
//
//	Variables for style information tables
//
int	nStdFont;	// Number of Standard Font registered
int	nExtFont;	// Number of Extended Font registered
char	szStdStyle[MAXSTDFONT][STYLENAMESIZE];	// Sorted by Font
char	szExtStyle[MAXEXTFONT][STYLENAMESIZE];	// Sorted by Font
char	szStdFont[MAXSTDFONT][FONTNAMESIZE];	// Sorted by Font
char	szStdFile[MAXSTDFONT][FILENAMESIZE];	// Sorted by Font
char	szExtFont[MAXEXTFONT][FONTNAMESIZE];	// Sorted by Font
char	szExtFile[MAXSTDFONT][FILENAMESIZE];	// Sorted by Font
char	szStdStylez[MAXSTDFONT][STYLENAMESIZE]; // Sorted by Style
char	szExtStylez[MAXEXTFONT][STYLENAMESIZE]; // Sorted by Style
//
#define VCREATE 	1	/* Create user variable */
#define VNORMAL 	0	/* R/W, saved with drawing */
#define V_INT		0	/* Integer attribute */
//
char	szTextEx[80]="Click Here to Enter Text!";       // From szText[1]
//
char	szBtn[10][20];	// For all common buttons (8 chars)
char	szText[30][60]; // Our display text buffer, see SetMsgText()
			// at the end of this file
//
//	The Following are predefined values for user direct selection.
//	The local technical supporter may modify and add entries to
//	these tables for those values commonly used by the operator.
//
char	szHeightTable[][6]={	// Height Factors
	"20.0","15.0","12.0","10.0","8.0","6.0","5.0","3.0","2.0"
};
char	szWidthTable[][6]={	// Width Factors
	"2.0","1.5","1.25","1.0","0.9","0.8","0.75","0.6","0.5"
};
char	szAngleTable[][6]={	// Oblique Angles
	"+45.","+30.","+15.","0","-15.","-30","-45"
};
char	szSpaceTable[][10]={	// Character Space
	"+5,Abs","+3,Abs","+1,Abs","0","-0.1","-0.2","-0.25","-0.5","-1.0"
};
char	szRowTable[][10]={	// Row space
	"+10,Abs","+5,Abs","+3,Abs","+1,Abs","By Text","-0.5","-1.0","-1.5","-2.0"
};
//
//	Variable for saving system variables
//
//	   The following variables are initialized with the content of
//	   those system variables that may be modified during program
//	   execution. They are used at program termination to setup the
//	   undo information of these system variables. In fact, a more
//	   simple way is to issue set_undo("var") directly at the program
//	   start-up. However, the technique shown here will reduce the
//	   overhead of the undo buffer by issing set_undo() only for
//	   those variables that do have been modified after user
//	   operation.
//
double	sCharWidth   =	CharWidth;
double	sCharHeight  =	CharHeight;
double	sCharSpace   =	CharSpace;
double	sCharObAngle =	CharObAngle;
double	sRowSpace    =	RowSpace;
int	sCharRot90   =	CharRot90;
int	sCharObx     =	CharObx;
char	sStdSname[80] = szStdSname;
char	sExtSname[80] = szExtSname;
//
//	Variable for global usage
//
int	widMain;	// Main Dialog Window ID
int	widVport;	// Sub Window for text view port
int	nMain[6]={4,22|0x800,3,77,0x3270,0x8000};
int	nVport[5]={3,7,1,60,0x32f7};
int	nTatrb=0x270;
int	nSatrb=0x270;
int	nStdIndex;	// Current selected standard style index
int	nExtIndex;	// Current selected extended style index
int	nStdLoad=-1;	// Last loaded standard style
int	nExtLoad=-1;	// Last loaded extended style
int	mUndoStd=-1;	// Mark to undo standard style
int	mUndoExt=-1;	// Mark to undo extended style
int	mUndoAll=-1;	// Mark to undo all
int	nFwidth=62;

#define HASTEXT 	1
#define HASLAST 	2

int	nFlag=0;	// Our general flag
double	xWidth; 	// Text Width
//
//	Variable for Button ID used globally
//
int	bText;
int	bStyle; 	// Change Style
int	bCancel;	// Cancel buttons
int	bOk;		// Ok Buttons
int	bHelp;		// Help buttons
int	bToler; 	// call Tolerstr
int	bPickup;	// Pick existing
int	bRebuild;	// Rebuild style information
int	bRestore;	// Restore entity
//
int	bStdStyle;	// Standard Style Name BID
int	bStdFont;	// Standard Style Font Name BID
int	bExtStyle;	// Extended Style Name BID
int	bExtFont;	// Extended Style Font Name BID
int	bTxHeight;	// Text Height BID
int	bTxWidth;	// Text Width Factor BID
int	bObAngle;	// Text Oblique Angle BID
int	bChSpace;	// Text Character Spacing BID
int	bRowSpace;	// Text Row Default Spacing BID
int	bTxRot90;	// Font Rotated 90 Degree BID
int	bTxObx; 	// Oblique From Base line BID
//
int	bTop, bHalf, bMiddle, bBase, bBottom;
int	bLeft, bCenter, bRight;
int	bAlign, bFit, bEven;
int	bPnt, bEnt;
//
//	Do we have direct support of some new TCL functions...
//
int	nSupport=identify("wnd_resize");
int	IsWin;		// Is Windows Version?

COMMAND ChText()
{
	SLIST	slist;
	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}

	SetMsgText();	// Set Message Text, see end of this program

	slist = getesel(szText[14]/*"\nSelect Text to modify (%s): "*/,FTEXT);
	switch(ent_count(slist)) {
	  case 0: return;	// do nothing
	  case 1: DoSingle(slist[0]);	return;
	  default:
		DoMass(slist);
		break;
	}

}

//
#define MDSTDSTYLE	1
#define MDEXTSTYLE	2
#define MDTXHEIGHT	4
#define MDTXWIDTH	8
#define MDTXOB		0x10
#define MDTXCSPACE	0x20
#define MDTXROTATE	0x40
//
ENTITY	eTarget;

void	DoMass(SLIST slist)
{
	char	szBuf[128];
	int	cmdecho;
	int	nFlag;
	int	nCount;
	ENTDATA data;
	int	nHasChange;
	double	obAngle;
	int	nStdID, nExtID;
	int	i;

	mUndoAll = set_undo(1);

	eTarget = slist[0];
	Initialize();	// Do the first time initialization
	DoPickup(0);

	exbuf_io(0,"%%DDSTYLE -MODIFY",128);

	cmdecho = @cmdecho;
	@cmdecho = 0;
	DoCommand("DDSTYLE");         // Go for it!
	@cmdecho = cmdecho;

	exbuf_io(1,szBuf,sizeof(szBuf));	// Get return message
	if ( szBuf == "##DDSTYLE CANCEL" ) {
		ResetVariables();
		return;
	}
	if ( left$(szBuf,12) != "##DDSTYLE OK" ) {
		undo(mUndoAll); // Undo everthing
		ResetVariables();
		return;
	}
	szBuf -= "##DDSTYLE OK";
	szBuf -= " ";
	nFlag = eval(szBuf+"+0");
	if ( nFlag == 0 ) {
		undo(mUndoAll); // Undo everthing
		ResetVariables();
		return;
	}

	obAngle = CharObAngle;
	if ( CharRot90 && (abs(obAngle)<(pi/2))) {
		if ( obAngle >= 0 ) obAngle += pi/2;
		else		    obAngle -= pi/2;
	}
	if ( CharObx && (abs(obAngle)<pi)) {
		if ( obAngle > 0 )     obAngle = 2*pi - obAngle;
		else if ( obAngle < 0) obAngle = -2*pi- obAngle;
	}

	nStdID = styleid(szStdSname);
	nExtID = styleid(szExtSname);
	nCount = ent_count(slist);
	for(i=0;i<nCount;i++) {
		nHasChange = 0;
		eTarget = slist[i];
		data = ent_read(eTarget);
	   if ( nFlag & MDTXHEIGHT )
		if ( data.text.height != CharHeight )
		{	data.text.height = CharHeight;	nHasChange++;	}
	   if ( nFlag & MDTXOB )
		if ( data.text.obangle != obAngle )
		{	data.text.obangle = obAngle;	 nHasChange++;	}
	   if ( nFlag & MDTXCSPACE )
		if ( data.text.csp != CharSpace )
		{	data.text.csp = CharSpace;	 nHasChange++;	}
	   if ( nFlag & MDTXWIDTH )
		if ( data.text.width != CharWidth )
		{	data.text.width = CharWidth;	 nHasChange++;	}
	   if ( nFlag & MDSTDSTYLE )
		if ( szStdSname != style(data.text.styleno).name )
		{	data.text.styleno = nStdID;  nHasChange++; }
	   if ( nFlag & MDEXTSTYLE )
		if ( szExtSname != style(data.text.estyleno).name )
		{	data.text.estyleno = nExtID; nHasChange++; }
		if ( nHasChange )
			ent_write(eTarget,data);
	}
	ResetVariables();
}
//
//	Single entity selected, so the operator may change its text
//	content and everything.
//
void	DoSingle(ENTITY ent)
{
	eTarget = ent;

	Initialize();	// Do the first time initialization
			// If anything wrong, it won't return here
	mUndoAll = set_undo(1);

	DoPickup(0);
	userbrk(0);	// Turn off User Break
	for(;;) {
		switch(DoDialog()) {
		  case DOPICKUP:
			DoPickup(1);
			continue;
		  case DOIT:
			DoUpdate();
			ResetVariables();
			break;
		  default:		// Cancel
			undo(mUndoAll); // Undo everthing
			ResetVariables();
			break;
		}
		break;
	}
}
//
void	DoUpdate()
{
	ENTDATA data;
	int	nHasChange;
	double	obAngle;

	obAngle = CharObAngle;
	if ( CharRot90 && (abs(obAngle)<(pi/2))) {
		if ( obAngle >= 0 ) obAngle += pi/2;
		else		    obAngle -= pi/2;
	}
	if ( CharObx && (abs(obAngle)<pi)) {
		if ( obAngle > 0 )     obAngle = 2*pi - obAngle;
		else if ( obAngle < 0) obAngle = -2*pi- obAngle;
	}

	nHasChange = 0;
	data = ent_read(eTarget);
	if ( data.text.str != szTextEx ) {
		if ( nFlag & HASTEXT ) {
			data.text.str = szTextEx;
			nHasChange++;
		}
	}

	if ( data.text.height != CharHeight )
	{	data.text.height = CharHeight;	nHasChange++;	}
	if ( data.text.obangle != obAngle )
	{	data.text.obangle = obAngle;	 nHasChange++;	}
	if ( data.text.csp != CharSpace )
	{	data.text.csp = CharSpace;	 nHasChange++;	}
	if ( data.text.width != CharWidth )
	{	data.text.width = CharWidth;	 nHasChange++;	}
	if ( szStdSname != style(data.text.styleno).name )
	{	data.text.styleno = styleid(szStdSname);  nHasChange++; }
	if ( szExtSname != style(data.text.estyleno).name )
	{	data.text.estyleno = styleid(szExtSname); nHasChange++; }

	if ( nHasChange )
		ent_write(eTarget,data);
}
//
void	ResetVariables()
{
	CharWidth   = sCharWidth;
	CharHeight  = sCharHeight;
	CharSpace   = sCharSpace;
	CharObAngle = sCharObAngle;
	RowSpace    = sRowSpace;
	CharRot90   = sCharRot90;
	CharObx     = sCharObx;
	szStdSname  = sStdSname;
	szExtSname  = sExtSname;
	style(szStdSname,0);	     // ReActivate the style
	style(szExtSname,1);	     // ReActivate the style
}
//
int	DoDialog()
{
	int	nBtn;
	int	nRet;
	char	szTemp[128];

	OpenDialog();
	UpdateDialog();
	nRet = DOCANCEL;
	for(;;) {
		nBtn = wnd_gbtn(0);
		switch(nBtn) {
		  case -3:
		  case -2:
		  case -1:
		  case	0:	nRet = DOCANCEL;  break;
		  case bRestore:	DoPickup(0);
					UpdateDialog(); continue;
		  case bRebuild:	RebuildTable(); continue;
		  case bStdStyle:	SelStdStyle();	continue;
		  case bStdFont:	SelStdFont();	continue;
		  case bExtStyle:	SelExtStyle();	continue;
		  case bExtFont:	SelExtFont();	continue;
		  case bTxHeight:	SetTextHeight(); continue;
		  case bTxWidth:	SetTextWidth(); continue;
		  case bObAngle:	SetObAngle();	continue;
		  case bChSpace:	SetCharSpace(); continue;
		  case bTxRot90:	ToggleRot90();	continue;
		  case bTxObx:		ToggleObx();	continue;
		  case bStdStyle+1:	SelStdStyle();	continue;
		  case bStdFont+1:	SelStdFont();	continue;
		  case bExtStyle+1:	SelExtStyle();	continue;
		  case bExtFont+1:	SelExtFont();	continue;
		  case bTxHeight+1:	SelTextHeight();continue;
		  case bTxWidth+1:	SelTextWidth(); continue;
		  case bObAngle+1:	SelObAngle();	continue;
		  case bChSpace+1:	SelCharSpace(); continue;
		  case bText:
			szTextEx = wnd_fgets(bText,72,0x44e,nFlag&HASTEXT?szTextEx:"");
			if ( szTextEx != "" )
				nFlag |= HASTEXT;
			else {
				nFlag &= ~HASTEXT;
				szTextEx = szText[1];
			}
			UpdateDialog();
			continue;
		  case bToler:
			DoTolerstr();
			continue;
		  case bPickup:
			nRet = DOPICKUP;
			break;
		  case bOk:	nRet = DOIT;	break;
		  case bCancel: nRet = -1;	break;
		  case 0x2308:	// ^H
		  case bHelp:
			show_help("","CHTEXT");
			continue;
		  default:
			continue;
		}
		break;
	}
	CloseDialog();
	return nRet;
}
//
void	RebuildTable()
{
	int	cmdecho;
	exbuf_io(0,"%%DDSTYLE -REBUILD",128);

	cmdecho = @cmdecho;
	@cmdecho = 0;
	DoCommand("DDSTYLE");         // Go for it!
	@cmdecho = cmdecho;

	if ( !LoadTable() ) {
		printf("\n## DDSTYLE fails to locate any font file.");
		exit(1);
	}
	UpdateTable();
	UpdateDialog();
}
//
//	OpenDialog() -- Open main dialog window, create screen items.
//
void	OpenDialog()
{
	int	rBase, cBase, cOfst, rSave;
	int	rInc=0x0801;
	widMain = wndopen(nMain,szText[0] /* "Text Style Setup Dialog" */,0x3f1f,nMain[5]);
	rBase = 0x0801;
	cBase = 1;
	nFwidth = (nVport[3]-nVport[2])+1;
	WndGroup(rBase-1,nVport[2],72,1,szText[2]/*"New Text & Preview"*/,nSatrb);
	bText = WndEntry(rBase+0x0100,cBase,"",72|0x100,0);
	WndGroup(nVport[0]-1,nVport[2],nFwidth,
		(nVport[1]&0xff)-(nVport[0]&0xff)+1,"",nSatrb);
	widVport = wndopensub(nVport,0);   // Open sub window for image
	wnd_select(widMain);
	rBase = nVport[1]+0x0801;
	cBase = nVport[2];

	WndGroup(rBase,cBase,nFwidth,3,szText[3]/*"Font Style Setup"*/,nSatrb);
	rBase += 0x0401;
	cBase = 4;

	bStdStyle = WndEntry(rBase+rInc*0,cBase,szText[4]/*"Standard "*/,0x109,1);  // "Standard "
	bStdFont  = WndEntry(rBase+rInc*0,cBase+22,""    ,0x11c,1);
	bExtStyle = WndEntry(rBase+rInc*1,cBase,szText[5]/*"Extended "*/,0x109,1);  // "Extended "
	bExtFont  = WndEntry(rBase+rInc*1,cBase+22,""    ,0x11c,1);

		wnd_rloct(0xfc00,IsWin?0x01:0x02);
		bRebuild = wnd_field(0,IsWin?0x277:0x370,0x201);
		if ( IsWin )
			wnd_ftext(bRebuild,"",0x277);
	rSave = rBase;
	rBase += 0x0403;
	WndGroup(rBase,nVport[2],nFwidth,3,szText[6]/*"Text Attribute"*/,nSatrb);
	rBase += 0x0401;
	cBase = 1;
	bTxHeight= WndEntry(rBase+rInc*0,cBase,szText[7]/*" Height: "*/,0x108,1);  // "Defaut Height: "
	bTxWidth = WndEntry(rBase+rInc*1,cBase,szText[8]/*"  Width: "*/,0x108,1);  // " Width Factor: "
	cBase += len(szText[7])+11;
	bObAngle = WndEntry(rBase+rInc*0,cBase,szText[9]/*"Oblique: "*/,0x108,1);  // "Oblique Angle: "
	bChSpace = WndEntry(rBase+rInc*1,cBase,szText[10]/*"Spacing: "*/,0x108,1);  // " Char Spacing: "
	cBase += len(szText[9])+11;
	bTxObx	 = WndCheckBox(rBase+rInc*0,cBase,szText[11]/*"From X Base"*/,nSatrb); // "Oblique from X-Base "
	bTxRot90 = WndCheckBox(rBase+rInc*1,cBase,szText[12]/*"Font Rotated"*/,nSatrb); // "Rotate Font 90 Deg "
	rBase = nVport[0]+1;
	cBase = nVport[3]+4;
	rInc = 0x0402;
	bToler	 = WndButton(rBase,cBase,0x108,szBtn[6]/*&Toler.*/);
	bPickup  = WndButton(rBase+rInc*1,cBase,0x108,szBtn[7]/*&Sample*/);
	bRestore = WndButton(rBase+rInc*2,cBase,0x108,szBtn[8]/*&Restore*/);
	bOk	 = WndButton(rBase+rInc*3,cBase,0x108,szBtn[0]/*"&Ok"*/);
	bCancel  = WndButton(rBase+rInc*4,cBase,0x108,szBtn[1]/*"&Cancel"*/);
    if ( IsWin )
	bHelp	 = WndButton(rBase+rInc*5,cBase,0x108,szBtn[2]/*"&Help"*/);

}
//
//	Call Tolerstr to enter tolerance expression
//
void	DoTolerstr()
{
	char	szBuf[128];
	int	cmdecho;
	//
	// Now, we shall call DDSTYLE to setup text style control for the
	// text to create. A simple program interface is deviced using
	// the exbuf_io() function. See DDSTYLE.TCL for details.
	//

	exbuf_io(0,szTextEx,sizeof(szTextEx));	// Set return result

	cmdecho = @cmdecho;
	@cmdecho = 0;
	DoCommand("TOLERSTR");       // Go for it!
	@cmdecho = cmdecho;

	exbuf_io(1,szTextEx,sizeof(szTextEx));	// Get return result

	style(szStdSname,0);	     // ReActivate the style
	style(szExtSname,1);	     // ReActivate the style

	wnd_ftext(bText,szTextEx,nTatrb);
	UpdatePreview();
}
//
//	Execute a command (internal, extended or external)
//
//	TwinCAD Lite can not execute RUN command!
//
int	DoCommand(char cmdname[])
{
	if ( identify(cmdname) )	// Is command exist?
		return command(cmdname);    // Yes, do it!
	if ( identify("SystemMode") ) {
		if ( SystemMode() >= 0 )   // Is it non-lite mode?
			return command("RUN "+cmdname);  // Yes, run the name file
		printf("\nCan not execute %s command!",cmdname);
		return 0;
	}
	return command("RUN "+cmdname); // Execute it anyway!
}
//
//	CloseDialog() -- Close the main dialog window, save window attribute
//
void	CloseDialog()
{
	if ( IsWin ) {
	    if ( wnd_getsize(nMain) ) {
		put_profile(SECTIONNAME,MAINWINDOW,
			prints("%xh,%xh,%xh,%xh,%xh",nMain[0],nMain[1],
				nMain[2],nMain[3],nMain[4]));
	    }
	}
	wndclose();
}
//
//	Pickup exising text for default text, style, attributes
//
void	DoPickup(int type)
{
	ENTDATA data;

	if ( type ) {
		e1=gete(szText[13]/*"\nPlease select the text for style setting: "*/,FTEXT);
		if ( !ent_ok(e1) ) return;
		data = ent_read(e1);
	}
	else {
		data = ent_read(eTarget);
	}
	if ( !(nFlag & HASTEXT) || !type ) {
		szTextEx = data.text.str;    nFlag |= HASTEXT;
	}
	CharHeight = data.text.height;
	CharObx = CharRot90 = 0;
	CharObAngle = mod(data.text.obangle,pi*2);
	if ( abs(CharObAngle)>=pi ) {
		CharObx = 1;
		if ( CharObAngle > 0 )	CharObAngle = 2*pi - CharObAngle;
		else			CharObAngle = -2*pi - CharObAngle;
	}
	if ( abs(CharObAngle)>=(pi/2) ) {
		CharRot90 = 1;
		CharObAngle = mod(CharObAngle,pi/2);
	}
	CharSpace  = data.text.csp;
	CharWidth  = data.text.width;
	if ( data.text.styleno >= 0 )
		szStdSname = style(data.text.styleno).name;
	if ( data.text.estyleno >= 0 )
		szExtSname = style(data.text.estyleno).name;
	style(szStdSname,0);	     // ReActivate the style
	style(szExtSname,1);	     // ReActivate the style
	nStdIndex = search(szStdSname,szStdStyle,nStdFont);
	nExtIndex = search(szExtSname,szExtStyle,nExtFont);
}
//
//	UpdateDialog() -- Update the content of the main dialog window
//
//	Note:	The use of wnd_fmark to make a check on the checkbox
//		is not fully compatible with DOS version, although
//		the expression: wnd_fmark(n,0,(Check)?0x8000:0x80ff)
//		still works.
//
void	UpdateDialog()
{
	wnd_ftext(bStdStyle,prints("%-9s",szStdStyle[nStdIndex]),nTatrb);
	wnd_ftext(bStdFont, prints("%-28s",szStdFont[nStdIndex]),nTatrb);
	wnd_ftext(bExtStyle,prints("%-9s",szExtStyle[nExtIndex]),nTatrb);
	wnd_ftext(bExtFont, prints("%-28s",szExtFont[nExtIndex]),nTatrb);

	wnd_ftext(bTxHeight,prints("%8.4f",CharHeight),DATACOLOR);
	wnd_ftext(bTxWidth, prints("%8.4f",CharWidth),DATACOLOR);
	wnd_ftext(bObAngle, prints("%8.4f",rtd(CharObAngle)),DATACOLOR);
	wnd_ftext(bChSpace, prints("%8.4f",CharSpace),DATACOLOR);

	wnd_fmark(bTxRot90,0,CharRot90?MARKON:MARKOFF);
	wnd_fmark(bTxObx,  0,CharObx?MARKON:MARKOFF);
	if ( szTextEx == "" )
		szTextEx = " ";
	wnd_ftext(bText,szTextEx,nTatrb);
	UpdatePreview();
}
//
void	ToggleRot90()
{
	CharRot90 = !CharRot90;
	wnd_fmark(bTxRot90,0,CharRot90?MARKON:MARKOFF);
	UpdatePreview();
}
//
void	ToggleObx()
{
	CharObx = !CharObx;
	wnd_fmark(bTxObx,  0,CharObx?MARKON:MARKOFF);
	UpdatePreview();
}
//
//	SetTextHeight() -- Change Default Text Height
//
void	SetTextHeight()
{
	CharHeight = wnd_fgetd(bTxHeight,8,0x2e0,CharHeight);
	wnd_ftext(bTxHeight, prints("%8.4f",CharHeight),DATACOLOR);
	UpdatePreview();
}
//
//	SelTextHeight() -- Select Default Text Height
//
void	SelTextHeight()
{
	int	which;
	double	dval;
	if ( (which=wnd_fgeto(bTxHeight,szHeightTable,0xf0,6,-1))<0 )
		return;
	sscanf(szHeightTable[which]," %f",&dval);
	CharHeight = dval;
	wnd_ftext(bTxHeight, prints("%8.4f",CharHeight),DATACOLOR);
	UpdatePreview();
}
//
//	SetTextWidth() -- Change Text Width Factor
//
void	SetTextWidth()
{
	CharWidth = wnd_fgetd(bTxWidth,8,0x2e0,CharWidth);
	wnd_ftext(bTxWidth, prints("%8.4f",CharWidth),DATACOLOR);
	UpdatePreview();
}
//
//	SelTextWidth() -- Select Predefined Text Width Factor
//
void	SelTextWidth()
{
	int	which;
	double	dval;
	if ( (which=wnd_fgeto(bTxWidth,szWidthTable,0xf0,6,-1))<0 )
		return;
	sscanf(szWidthTable[which]," %f",&dval);
	CharWidth = dval;
	wnd_ftext(bTxWidth, prints("%8.4f",CharWidth),DATACOLOR);
	UpdatePreview();
}
//
//	SetObAngle() -- Change Text Oblique Angle
//
void	SetObAngle()
{
	CharObAngle = dtr(wnd_fgetd(bObAngle,8,0x2e0,rtd(CharObAngle)));
	wnd_ftext(bObAngle, prints("%8.4f",rtd(CharObAngle)),DATACOLOR);
	UpdatePreview();
}
//
//	SelObAngle() -- Select Text Oblique Angle
//
void	SelObAngle()
{
	int	which;
	double	dval;
	if ( (which=wnd_fgeto(bObAngle,szAngleTable,0xf0,6,-1)) < 0 )
		return;
	sscanf(szAngleTable[which]," %f",&dval);
	CharObAngle = dtr(dval);
	wnd_ftext(bObAngle, prints("%8.4f",rtd(CharObAngle)),DATACOLOR);
	UpdatePreview();
}
//
//	SetCharSpace() -- Change Character Space
//
void	SetCharSpace()
{
	CharSpace = wnd_fgetd(bChSpace,8,0x2e0,CharSpace);
	wnd_ftext(bChSpace, prints("%8.4f",CharSpace),DATACOLOR);
	UpdatePreview();
}
//
//	SelCharSpace() -- Select Character Space
//
void	SelCharSpace()
{
	int	which;
	double	dval;
	if ( (which=wnd_fgeto(bChSpace,szSpaceTable,0xf0,6,-1)) < 0 )
		return;
	sscanf(szSpaceTable[which]," %f",&dval);
	CharSpace = dval;
	wnd_ftext(bChSpace, prints("%8.4f",CharSpace),DATACOLOR);
	UpdatePreview();
}
//
//	SelStdStyle() -- Select Standard Style by Style Name
//
void	SelStdStyle()
{
//	nStdIndex = wnd_fgeto(bStdStyle,szStdStyle,0xf0,8,nStdIndex);

	int	idx;
	idx = search(szStdStyle[nStdIndex],szStdStylez,nStdFont);
	idx = wnd_fgeto(bStdStyle,szStdStylez,0xf0,8,idx);
	nStdIndex = search(szStdStylez[idx],szStdStyle,nStdFont);
	UpdStdStyle();
}
//
//	SelStdStyle() -- Select Standard Style by Font Name
//
void	SelStdFont()
{
	nStdIndex = wnd_fgeto(bStdFont,szStdFont,0xf0,8,nStdIndex);
	UpdStdStyle();
}
//
//	SelExtStyle() -- Select Extended Style by Style Name
//
void	SelExtStyle()
{
//	nExtIndex = wnd_fgeto(bExtStyle,szExtStyle,0xf0,8,nExtIndex);
	int	idx;
	idx = search(szExtStyle[nExtIndex],szExtStylez,nExtFont);
	idx = wnd_fgeto(bExtStyle,szExtStylez,0xf0,8,idx);
	nExtIndex = search(szExtStylez[idx],szExtStyle,nExtFont);
	UpdExtStyle();
}
//
//	SelExtStyle() -- Select Extended Style by Font Name
//
void	SelExtFont()
{
	nExtIndex = wnd_fgeto(bExtFont,szExtFont,0xf0,8,nExtIndex);
	UpdExtStyle();
}
//
//	UpdStdStyle() -- Update the content of the standard style
//			 and load the style
//
void	UpdStdStyle()
{
	wnd_ftext(bStdStyle,prints("%-9s",szStdStyle[nStdIndex]),nTatrb);
	wnd_ftext(bStdFont, prints("%-28s",szStdFont[nStdIndex]),nTatrb);
	LoadStdStyle();
	UpdatePreview();
}
//
//	UpdExtStyle() -- Update the content of the extended style
//			 and load the style
//
void	UpdExtStyle()
{
	wnd_ftext(bExtStyle,prints("%-9s",szExtStyle[nExtIndex]),nTatrb);
	wnd_ftext(bExtFont, prints("%-28s",szExtFont[nExtIndex]),nTatrb);
	LoadExtStyle();
	UpdatePreview();
}
//
//	LoadStdStyle() -- Make sure the current standard style is loaded.
//
void	LoadStdStyle()
{
	if ( nStdIndex == nStdLoad )	// No change,
		return;
	if ( mUndoStd != -1 ) { 	// Yes, have undo mark
		undo(mUndoStd);
		if ( mUndoStd < mUndoExt ) {
			nStdLoad = -1;
			mUndoStd = -1;
			nExtLoad = -1;
			mUndoExt = -1;	// Extended style also unloaded
			UpdExtStyle();	// So, load it again
		}
	}
	mUndoStd = set_undo(1);
	nStdLoad = nStdIndex;
	style(szStdStyle[nStdIndex],0,szStdFile[nStdIndex]);
	szStdSname = szStdStyle[nStdIndex];
}
//
//	LoadExtStyle() -- Make sure the current extended style is loaded.
//
void	LoadExtStyle()
{
	char	szSHE[124];
	char	szSHD[124];
	char	szFname[124];
	char	szExe[124];
	char	szParm[256];
	int	nType;

	if ( nExtIndex == nExtLoad )	// No change,
		return;
	if ( mUndoExt != -1 ) { 	// Yes, have undo mark
		undo(mUndoExt);
		if ( mUndoExt < mUndoStd ) {
			nStdLoad = -1;
			mUndoStd = -1;
			nExtLoad = -1;
			mUndoExt = -1;	// Standard style also unloaded
			UpdStdStyle();	// So, load it again
		}
	}
	mUndoExt = set_undo(1);
	nExtLoad = nExtIndex;
	szSHE = takename(szExtFile[nExtIndex])-("."+left$(szExtFont[nExtIndex],3))+".SHE";
	szFname = fsearch(szSHE);	// Locate the file
	if ( szFname != "" ) {
		style(szExtStyle[nExtIndex],1,szFname);
		return;
	}
	// Not found, must create one
	nType = get_fontinf(szExtFile[nExtIndex]);
	szExe = "MAKESHE";
	switch(nType) {
	  case BIG5:	szSHD = "BIG5TTFX.SHD"; szExe="BIG5TTF"; break;
	  case GB2312:	szSHD = "GBTTF.SHD";    szExe="GBTTF";   break;
	  case SJIS:	szSHD = "SJISTTF.SHD";  szExe="SJISTTF"; break;
	  case KOREAN:	szSHD = "MSCTTF.SHD";   break;
	  case BIGSHX:	szSHD = "BSHXZ.SHD";    break;
	  default:
		printf("\nDon't know how to process %s",szExtFile[nExtIndex]);
		return;
	}
	szFname = fsearch(szSHD);
	if ( szFname == "" ) {
		printf("\nCan not locate font driver %s",szSHD);
		return;
	}
	szSHD = szFname;
	szFname = fsearch(szExe+".PIF");        // Find PIF first
//
//	if ( szFname == "" )
//		szFname = fsearch(szExe+".EXE");
//
//	The file searching will set the current drive implicitly in Win, so
//	we must search for the EXE from our path, and make the path as
//	the current path and drive, so that Windows can locate it from
//	the use of the PIF.
//
	szExe = fsearch(szExe+".EXE");
//

	if ( szFname != "" )
		szExe = szFname;

	switch(nType) {
	  case BIG5:
	  case GB2312:
	  case SJIS:
		szParm = szExtFile[nExtIndex]+" "+szSHE+" "+szSHD+" -q";
		break;
	  case BIGSHX:
		szParm = szExtFile[nExtIndex]+";* "+szSHD+" "+szSHE+" -q";
		break;		// Assuming Outline font
	  default:
		szParm = szExtFile[nExtIndex]+" "+szSHD+" "+szSHE+" -q";
	}

	printf("\nexec %s %s",szExe,szParm);

	exec(szExe,szParm,0);	// Run Background
	szFname = fsearch(szSHE);	// Locate the file again
	if ( szFname == "" ) {
		printf("\n## Fail to create the SHE file: %s",szSHE);
		return;
	}
	style(szExtStyle[nExtIndex],1,szFname);
	szExtSname = szExtStyle[nExtIndex];	// Fix for bug in .045 - .047-
}
//
//	UpdatePreview() -- Update Example Font in preview sub-window
//
void	UpdatePreview()
{
	double	xH;
	wnd_select(widVport);
	xWidth = text_span(szTextEx);
	xH = -0.75;
	if ( index(szTextEx,"^") ) {
		switch(TRUE) {
		  case RowSpace>0:	xH = -RowSpace/CharHeight-0.5;	break;
		  case RowSpace==0:	xH = -2.0;	break;
		  default:	xH = RowSpace-0.5;
		}
	}
	wnd_vport(-1);
	wnd_vport(0,p(-xWidth*0.05,CharHeight*1.75),p(xWidth*1.05,xH*CharHeight),1);
	setplot(1,0,0,0);
	plotent(szTextEx,p(0,0),0);
	wnd_vport(-1);
	wnd_select(widMain);
}
//
void	Initialize()
{
	char	szBuf[256];
	int	cmdecho;

	szTextEx = szText[1];	// Default message text
	// Determine the main dialog window position
	nTatrb = 0x2f1;
	if ( IsWin ) {	// Only in Windows version
		szBuf = get_profile(SECTIONNAME,MAINWINDOW);
		if ( szBuf != "" ) {    // Yes, not the first time
			int	nWidth;
			int	nHeight;
			nWidth=nMain[3]-nMain[2];
			nHeight=nMain[1]-nMain[0];
			sscanf(szBuf,"%xh, %xh, %xh, %xh, %xh",&nMain[0],&nMain[1],
				&nMain[2],&nMain[3],&nMain[4]);
			// Since we are using fix window
			nMain[1] = nMain[0]+nHeight;
			nMain[3] = nMain[2]+nWidth;
			nTatrb = nMain[4]&0xff;
			nMain[5] = 0;
		}
	}

	style(szStdSname,0);	     // ReActivate the style
	style(szExtSname,1);	     // ReActivate the style

	if ( !LoadTable() ) {
		printf("\n## Can not find style table, initialize DDSTYLE to build it...");
		exbuf_io(0,"%%DDSTYLE -REBUILD",128);

		cmdecho = @cmdecho;
		@cmdecho = 0;
		DoCommand("DDSTYLE");         // Go for it!
		@cmdecho = cmdecho;

		if ( !LoadTable() ) {
			printf("\n## DDSTYLE fails to locate any font file.");
			exit(1);
		}
	}
	UpdateTable();
	nStdIndex = search(szStdSname,szStdStyle,nStdFont);
	nExtIndex = search(szExtSname,szExtStyle,nExtFont);

}
//
void	UpdateTable()
{
	STYLE	st;
	int	i;

	st=style("",0);
	while((st.name!="")&&(nStdFont<MAXSTDFONT) ) {
		if ( search(st.name,szStdStyle,nStdFont)==-1) {
			szStdStyle[nStdFont] = st.name;
			szStdFile[nStdFont] = st.fname;
			szStdFont[nStdFont] = left$("Preload: "+st.name+
			   " from "+takename(st.fname),FONTNAMESIZE-1);
			nStdFont++;
		}
		st=tbl_next(st);
	}
	st = style("",1);
	while((st.name!="")&&(nExtFont<MAXEXTFONT)) {
		if ( search(st.name,szExtStyle,nExtFont)==-1) {
			szExtStyle[nExtFont] = st.name;
			szExtFile[nExtFont] = st.fname;
			szExtFont[nExtFont] = left$("Preload: "+st.name+
			   " from "+takename(st.fname),FONTNAMESIZE-1);
			nExtFont++;
		}
		st = tbl_next(st);
	}
	SortTable();
}
//
void	SortTable()
{
	int	nStdIdx[MAXSTDFONT];
	int	nExtIdx[MAXSTDFONT];

	setdata(szStdStylez,szStdStyle);
	setdata(szExtStylez,szExtStyle);
	sort(szStdStylez,nStdFont,0);
	sort(szExtStylez,nExtFont,0);

	// Sort the table by Font Name
	sort(szStdFont,nStdFont,0,nStdIdx);
	sort(szExtFont,nExtFont,0,nExtIdx);
	setdata(szStdStyle,szStdStyle,nStdIdx);
	setdata(szStdFile,szStdFile,nStdIdx);
	setdata(szExtStyle,szExtStyle,nExtIdx);
	setdata(szExtFile,szExtFile,nExtIdx);

	// Add Terminating string

	szStdStyle[nStdFont] = "";
	szStdFont[nStdFont] = "";
	szStdFile[nStdFont] = "";
	szExtStyle[nExtFont] = "";
	szExtFont[nExtFont] = "";
	szExtFile[nExtFont] = "";
}
//	==========================================================
//	LoadTable() -- Load font information table from data file
//
//	This routine is copied from DDSTYLE. See DDSTYLE.TCL for
//	more details.
//
//	Data File Structure:
//
//		long	FILEMAGICID;	// Checking code
//		int	nStdFont;
//		int	nExtFont;
//		int	nStyleSize;
//		int	nFontSize;
//		int	nFileSize;
//		char	szStdStyle[nStdFont][nStyleSize];
//		long	FILEMAGICID;	// Checking code
//		char	szStdFile[nStdFont][nFileSize];
//		long	FILEMAGICID;	// Checking code
//		char	szStdFont[nStdFont][nFontSize];
//		long	FILEMAGICID;	// Checking code
//		char	szExtStyle[nExtFont][nStyleSize];
//		long	FILEMAGICID;	// Checking code
//		char	szExtFile[nExtFont][nFileSize];
//		long	FILEMAGICID;	// Checking code
//		char	szExtFont[nExtFont][nFontSize];
//		long	FILEMAGICID;	// Checking code
//		EOF
//
#define DATAFILE	fullname("DDSTYLE.SET",get_syspath(0))
#define OLDDATAFILE	fullname("DDSTYLE.SET",get_syspath(1))
BOOL	LoadTable()
{
	int	hdl;
	long	lMagicID;
	int	nStyleSize, nFontSize, nFileSize;

	if ( !(hdl=fopen(DATAFILE,"r")) ) {
		if ( !(hdl=fopen(OLDDATAFILE,"r")) )
			return FALSE;
	}
	switch(FALSE) { // Evaluate cases sequentially in file data order...
	  case (fget(hdl,&lMagicID,&nStdFont,&nExtFont,
		 &nStyleSize,&nFontSize,&nFileSize)!=EOF):
	  case (lMagicID==FILEMAGICID):
	  case (nStdFont>0):
	  case (nExtFont>0):
	  case ReadString(hdl,szStdStyle,nStdFont,nStyleSize):
	  case ReadString(hdl,szStdFile,nStdFont,nFileSize):
	  case ReadString(hdl,szStdFont,nStdFont,nFontSize):
	  case ReadString(hdl,szExtStyle,nExtFont,nStyleSize):
	  case ReadString(hdl,szExtFile,nExtFont,nFileSize):
	  case ReadString(hdl,szExtFont,nExtFont,nFontSize):
		break;
	  default:	// If execution reach here...
		fclose(hdl);
		return TRUE;	// Table is good and read!
	}
	fclose(hdl);
	return FALSE;
}
//
//	ReadString() -- Read string array of specific size from file
//			with terminating code checking.
//
//	Return: True, if ok, False, if not.
//
BOOL	ReadString(int hdl, char szTable[][],int nCount, int nSize)
{
	long	lMagicID;
	char	szBuf[nCount][nSize];	// Allocate the require size
					// of the data array
	fget(hdl,szBuf,&lMagicID);
	if ( lMagicID != FILEMAGICID )
		return FALSE;		// Not Okay!
	setdata(szTable,szBuf); 	// Copy the string array
	szTable[nCount] = "";           // Add Terminating string
	return TRUE;
}
//
void	SetRadioMark(int bStart, int bEnd, int which)
{
	int	i;
	for(i=bStart;i<=bEnd;i++)
		wnd_fmark(i,1,((which+bStart)==i)?MARKON:MARKOFF);
}
//
void	SetBtnState(int bStart, int bEnd, int state)
{
	int	i;
	for(i=bStart;i<=bEnd;i++)
		wnd_state(i,1,state);
}
//
int	WndEntry(int row, int col, char str[],int fsize,int combo)
{
	int	bID;

	wnd_loct(row,col);
	put3Dstr(str,nSatrb);
	bID = wnd_field(0,IsWin?(DATACOLOR|0x100):0x21f1,fsize);
	if ( combo ) {
		wnd_rloct(0xfe00,0x0600|(fsize&0xff));
		if ( IsWin) wnd_runit(12);
		if ( identify("cb_empty") ) {   // Supported only after this version
		     wnd_icon(1,0xA77,0x101,OBM_COMBO);
		     wnd_runit(-1);
		     wnd_rloct(0x0200,0x0002);	// locate back
		}
		else {
		     wnd_field(0,IsWin?0x277:0x370,0x101);
		     wnd_runit(IsWin?-1:16);
		     wnd_ftext(bID+1,"",IsWin?0x277:0x270);
		     wnd_rloct(0x0200,0x0201);	// locate back
		}
	}
	return bID;
}
//
int	WndButton(int r,int c, int bsize, char text[])
{
	int	i;
	if ( nSupport ) 	// Having direct support ?
		return wnd_button(r,c,bsize,0x1277,text);
	wnd_loct(r,c);
	i = wnd_field(0,IsWin?0x1277:0x370,bsize);
	wnd_puts(text,0x270);
	return i;
}
//
//	WndGroup() -- Create Groupping window effect
//
void	WndGroup(int r,int c,int width, int height, char text[], int color)
{
	wnd_loct(r,c);
	wnd_rloct(height,0);
	if ( IsWin )
		wnd_field(0,0x277,(height<<8)|width);
	else
		wnd_field(0,((color&0xff)^0x08)|0x200,(height<<8)|width);
	wnd_itemno(1,-1);
	wnd_rloct(-height,2);
	if ( !IsWin )
		wnd_rloct(0xfc00,0);
	put3Dstr(text,color);
	wnd_loct(r,c);	      // Reset row offset, fix DOS version bug
}
//
//	put3Dstr() -- Drawing 3D text
//
void	put3Dstr(char str[],int atrb)
{
	int	nLen;
	if ( nSupport ) {
		wnd_puts(str,atrb|0x300);
		return;
	}
	nLen=len(str);
	if ( nLen ) {
		wnd_rloct(0x100,IsWin?0x100:0);
		wnd_puts(str,(atrb^0x0f));
		wnd_rloct(0xff00,-nLen);
		wnd_puts(str,(atrb&0xff0f)|0x8000);
	}
}
//
//	WndCheckBox() -- Create window check box
//
int	WndCheckBox(int r,int c,char text[],int atrb)
{
	int	bID;
	if ( nSupport )
		return wnd_rcbox(r,c,text,0);
	wnd_loct(r,c);
	bID = wnd_field(1,0x1f0,0x102);
	wnd_rloct(0,2);
	if ( IsWin )  atrb |= 0x800;
	wnd_puts(text,atrb);
	return bID;
}
//
//	WndRadioBox() -- Create window radio box
//
int	WndRadioBox(int r,int c,char text[],int atrb)
{
	int	bID;
	if ( nSupport )
		return wnd_rcbox(r,c,text,0x8000);
	wnd_loct(r,c);
	bID = wnd_field(1,0x1f0,0x102);
	wnd_rloct(0,2);
	if ( IsWin ) atrb |= 0x800;
	wnd_puts(text,atrb);
	wnd_fmark(bID,1,IsWin?1:MARKON);     // Force to change state
	wnd_fmark(bID,1,IsWin?0:MARKOFF);
	return bID;
}

char	szAsciiText[][54]={	    // Default English Display Text
/*  0 */   "Text Modification Dialog",
/*  1 */   "Click Here to Enter Text!",
/*  2 */   "Text Entry & Preview",
/*  3 */   "Text Style Fonts",
/*  4 */   "Standard ",
/*  5 */   "Extended ",
/*  6 */   "Text Attribute",
/*  7 */   " Height: ",
/*  8 */   "  Width: ",
/*  9 */   "Oblique: ",
/* 10 */   "Spacing: ",
/* 11 */   "From X Base",
/* 12 */   "Font Rotated",
/* 13 */   "\nPlease select the text to copy style: ",
/* 14 */   "\nSelect Text to modify (%s): ",
};

char	szBig5Text[][54]={	   // Default Chinese Big-5 Display Text
/*  0 */   "rקާ@ܵ",
/*  1 */   "BJr Click to Enter Text!",
/*  2 */   "rJεGw",
/*  3 */   "r]w",
/*  4 */   "зǭ^: ",
/*  5 */   "XRy: ",
/*  6 */   "rSʳ]w",
/*  7 */   "   r: ",
/*  8 */   " re: ",
/*  9 */   " rר: ",
/* 10 */   " rZ: ",
/* 11 */   "۹uɱ",
/* 12 */   "rα90",
/* 13 */   "\nпnѷӦrwqrϤ: ",
/* 14 */   "\nпܭnק諸r]νƿ^ (%s): ",
};

char	szGB2312Text[][54]={	// Default Chinese GB2312 Display Text
/*  0 */   "޸ĲԻ",
/*  1 */   "ȡ˴ Click to Enter Text!",
/*  2 */   "뼰Ԥ",
/*  3 */   "趨",
/*  4 */   "׼Ӣ: ",
/*  5 */   ": ",
/*  6 */   "趨",
/*  7 */   "   ָ: ",
/*  8 */   " ֿ: ",
/*  9 */   " б: ",
/* 10 */   " ּ: ",
/* 11 */   "Իб",
/* 12 */   "ת90",
/* 13 */   "\nѡȡҪͶͼԪ: ",
/* 14 */   "\nѡҪ޸ĵ֣ѡѡ (%s): ",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][54]={    // Default Japenese SJIS Display Text
/*  0 */   "eLXg̏C̐ݒ",
/*  1 */   "NbNăeLXg͂܂.",
/*  2 */   "eLXg̓͂ƕ\\",
/*  3 */   "eLXg̃tHgEX^C",
/*  4 */   "W: ",
/*  5 */   "g: ",
/*  6 */   "̑",
/*  7 */   ": ",
/*  8 */   "@: ",
/*  9 */   "X: ",
/* 10 */   "Ԋu: ",
/* 11 */   "X̊Ԋu",
/* 12 */   "̉]px(ccw)",
/* 13 */   "\nX^CRs[eLXgI: ",
/* 14 */   "\nCeLXgI(%s): ",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][54]={	// Default Korean Display Text
/*  0 */   "Text Modification Dialog",
/*  1 */   "Click Here to Enter Text!",
/*  2 */   "Text Entry & Preview",
/*  3 */   "Text Style Fonts",
/*  4 */   "Standard ",
/*  5 */   "Extended ",
/*  6 */   "Text Attribute",
/*  7 */   " Height: ",
/*  8 */   "  Width: ",
/*  9 */   "Oblique: ",
/* 10 */   "Spacing: ",
/* 11 */   "From X Base",
/* 12 */   "Font Rotated",
/* 13 */   "\nPlease select the text to copy style: ",
/* 14 */   "\nSelect Text to modify (%s): ",
};

char	szAsciiBtn[][10]={	   // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Quit",
/*  4 */   "&Style",
/*  5 */   "&Insert",
/*  6 */   "&Toler.",
/*  7 */   "Sam&ple",
/*  8 */   "&Restore",
};

char	szBig5Btn[][10]={	  // Default English Display Text
/*  0 */   "Tw[&O]",
/*  1 */   "[&C]",
/*  2 */   "[&H]",
/*  3 */   "hX[&Q]",
/*  4 */   "r[&S]",
/*  5 */   "J[&I]",
/*  6 */   "t[&T]",
/*  7 */   "ѷ[&P]",
/*  8 */   "_[&R]",
};

char	szGB2312Btn[][10]={	    // Default English Display Text
/*  0 */   "ȷ[&O]",
/*  1 */   "ȡ[&C]",
/*  2 */   "˵[&H]",
/*  3 */   "˳[&Q]",
/*  4 */   "[&S]",
/*  5 */   "[&I]",
/*  6 */   "[&T]",
/*  7 */   "[&P]",
/*  8 */   "ָ[&R]",
};

char	szSJISBtn[][16]={	  // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "ݾ[&C]",
/*  2 */   "[&H]",
/*  3 */   "I[&Q]",
/*  4 */   "[&S]",
/*  5 */   "}[&I]",
/*  6 */   "[&T]",
/*  7 */   "[&P]",
/*  8 */   "A[&R]",
};

char	szKoreanBtn[][10]={	    // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Quit",
/*  4 */   "&Style",
/*  5 */   "&Insert",
/*  6 */   "&Toler.",
/*  7 */   "Sam&ple",
/*  8 */   "&Restore",
};

char	szDOSAsciiBtn[][10]={	      // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Quit  ",
/*  4 */   "  Style ",
/*  5 */   " Insert ",
/*  6 */   " Toler. ",
/*  7 */   " Sample ",
/*  8 */   " Restore",
};

char	szDOSBig5Btn[][10]={	     // Default English Display Text
/*  0 */   "  Tw  ",
/*  1 */   "    ",
/*  2 */   "    ",
/*  3 */   "  hX  ",
/*  4 */   "  r  ",
/*  5 */   "  J  ",
/*  6 */   "  t  ",
/*  7 */   "  ѷ  ",
/*  8 */   "  _  ",
};

char	szDOSGB2312Btn[][10]={	     // Default English Display Text
/*  0 */   "  ȷ  ",
/*  1 */   "  ȡ  ",
/*  2 */   "  ˵  ",
/*  3 */   "  ˳  ",
/*  4 */   "    ",
/*  5 */   "    ",
/*  6 */   "    ",
/*  7 */   "    ",
/*  8 */   "  ָ  ",
};

char	szDOSSJISBtn[][10]={	     // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   "  ݾ ",
/*  2 */   "    ",
/*  3 */   "I",
/*  4 */   "    ",
/*  5 */   "  }  ",
/*  6 */   "    ",
/*  7 */   "   ",
/*  8 */   "  A  ",
};

char	szDOSKoreanBtn[][10]={	       // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Quit  ",
/*  4 */   "  Style ",
/*  5 */   " Insert ",
/*  6 */   " Toler. ",
/*  7 */   " Sample ",
/*  8 */   " Restore",
};

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

int	SetMsgText()
{
	char	szBuf[80];
	int	switched;

	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}

	if ( IsWin ) {
	    szBuf = get_profile("General Setup","AlternateLanguage");
	    switched = 0;
	    if ( szBuf!="" )
		switched = eval(szBuf);
	    setdata(szText,szAsciiText);
	    setdata(szBtn,szAsciiBtn);
	    if ( switched ) {
		switch(os_language()) {
		  case BIG5:	setdata(szText,szBig5Text);	break;
		  case GB2312:	setdata(szText,szGB2312Text);	break;
		  case SJIS:	setdata(szText,szSJISText);	break;
		  case KOREAN:	setdata(szText,szKoreanText);	break;
		  default:	setdata(szText,szAsciiText);	break;
		}
		switch(os_language()) {
		  case BIG5:	setdata(szBtn,szBig5Btn);     break;
		  case GB2312:	setdata(szBtn,szGB2312Btn);   break;
		  case SJIS:	setdata(szBtn,szSJISBtn);     break;
		  case KOREAN:	setdata(szBtn,szKoreanBtn);   break;
		  default:	setdata(szBtn,szAsciiBtn);    break;
		}
	    }
	}
	else {	// Switched or not, depends on TGF type in DOS version
	    setdata(szText,szAsciiText);
	    setdata(szBtn,szDOSAsciiBtn);
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szText,szBig5Text);     break;
	      case TGGB2312:  setdata(szText,szGB2312Text);   break;
	      case TGSJIS:    setdata(szText,szSJISText);     break;
	      case TGKOREAN:  setdata(szText,szKoreanText);   break;
	      default:	      setdata(szText,szAsciiText);    break;
	    }
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szBtn,szDOSBig5Btn);     break;
	      case TGGB2312:  setdata(szBtn,szDOSGB2312Btn);   break;
	      case TGSJIS:    setdata(szBtn,szDOSSJISBtn);     break;
	      case TGKOREAN:  setdata(szBtn,szDOSKoreanBtn);   break;
	      default:	      setdata(szBtn,szDOSAsciiBtn);    break;
	    }
	}
	return 0;
}
//
//	End of this Program file
//
